package com.hipme.cw;

import danger.audio.Meta;
import danger.app.Event;
import danger.ui.Container;

// Example use of ActiveListView derived from org.cymry.active.AListEx
/**
 * This class works around a bug in danger.ui.ActiveListView
 * that prevents it from allowing focus to pass to siblings
 * when the ActiveListView is not the only child in the container.
 *
 * You can put this class into your own package and use
 * it as "ActiveListView", as long as you do not do
 * "import danger.ui.*".  If you do that, you will have
 * to rename the class.
 *
 * In the DEVICE_MULTIPLE_WHEEL case, the whole
 * wheel count is consumed just getting out of the
 * ActiveListView control, and none are left over
 * for moving up or down in the Container.  That
 * makes the motion a little uneven, but it's not
 * so bad.
 */
class ActiveListView extends danger.ui.ActiveListView {

  boolean muted = false;

  private Object preMove() {
    muted = Meta.isIDMuted(Meta.SCROLL_LIMIT);
    if (!muted)
      Meta.muteID(Meta.SCROLL_LIMIT);
    return getFocusedItem();
  }

  private void postMoveUp(Object lastItem) {
    if (! muted)
      Meta.unmuteID(Meta.SCROLL_LIMIT);
    if (lastItem == getFocusedItem())
      ((Container)getParent()).reverseFocus();
  }

  private void postMoveDown(Object lastItem) {
    Meta.unmuteID(Meta.SCROLL_LIMIT);
    if (lastItem == getFocusedItem())
      ((Container)getParent()).advanceFocus();
  }

  public boolean eventWidgetDown(int inWhichWidget, Event event) {
    switch (inWhichWidget) {
    case Event.DEVICE_WHEEL_PAGE_DOWN: 
    case Event.DEVICE_ARROW_DOWN: 
    case Event.DEVICE_MULTIPLE_WHEEL: 
    case Event.DEVICE_WHEEL: 
      {
	Object lastItem = preMove();
	super.eventWidgetDown(inWhichWidget, event);
	postMoveDown(lastItem);
	return true;
      }
    case Event.DEVICE_ARROW_UP: 
      {
	Object lastItem = preMove();
	super.eventWidgetDown(inWhichWidget, event);
	postMoveUp(lastItem);
	return true;
      }
    default:
      return super.eventWidgetDown(inWhichWidget, event);
    }
  }

  public boolean eventWidgetUp(int inWhichWidget, Event event) {
    switch(inWhichWidget) {
    case Event.DEVICE_WHEEL_PAGE_UP:
    case Event.DEVICE_MULTIPLE_WHEEL:
    case Event.DEVICE_WHEEL:
      {
	Object lastItem = preMove();
	super.eventWidgetUp(inWhichWidget, event);
	postMoveUp(lastItem);
	return true;
      }
    default:
      return super.eventWidgetUp(inWhichWidget, event);
    }
  }

  public boolean eventKeyDown(char i ,Event event) {
    switch (event.data) {
              
    case ' ': 
      {
	Object lastItem = preMove();
	super.eventKeyDown(i, event);
	postMoveDown(lastItem);
	return true;
      }
    case '@': 
      {
	Object lastItem = preMove();
	super.eventKeyDown(i, event);
	postMoveUp(lastItem);
	return true;
      }
    }
    return false;
  }
}
