package com.hipme.cw;

// Example use of ActiveListView derived from org.cymry.active.AListEx

import danger.app.*;
import danger.util.*;
import danger.ui.ScreenWindow;
import danger.ui.TextField;
import danger.audio.Sound;
import danger.audio.Meta;
import danger.audio.Sequence;
import danger.audio.AudioManager;
import danger.audio.AudioException;

public class CW extends Application implements Commands {
  private CWWindow mWindow;

  public CW() {
    mWindow = new CWWindow();
  }

  public void resume() {
    mWindow.show();
  }

  void addItem(Object item) {
    mWindow.mExampleList.addItem(item);
  }

  public final boolean receiveEvent(Event event) {
    boolean consumed = false;
    switch (event.type) {
    case kReturnToLauncher:
      returnToLauncher();
      consumed = true;
      break;
    }
    return consumed || super.receiveEvent(event);
  }
}

class CWWindow extends ScreenWindow implements Commands {

  public CWWindow() {
    setTitle("CW Example");

    {
      mExampleList = new ActiveList();
      mExampleList.addItem("CQ CQ");
      mExampleList.addItem("DE");
      mExampleList.addItem("WA5ZNU");
      mExampleList.addItem("R");
      mExampleList.addItem("K");
    }

    {
      mExampleListView = new MyActiveListView();
      mExampleListView.setList(mExampleList);
      mExampleListView.setPosition(10,10);
      mExampleListView.setSize(220,80);
      mExampleListView.setItemHeight(10);
      mExampleListView.setHasScrollbar(true);
      addChild(mExampleListView);
    }

    {
      mInputLine = new MyInputLine();
      mInputLine.setPosition(10, 100);
      mInputLine.setSize(220, 15);
    }

    {
      addChild(mInputLine);
      mInputLine.show();
      mExampleListView.show();
      setFocusedChild(mExampleListView);
    }
  }
  
  public boolean eventWidgetDown(int i, Event event) {
    boolean consumed = false;
    switch(i) {
    case Event.DEVICE_BUTTON_BACK:
      new Event(Application.getCurrentApp(), kReturnToLauncher).sendToTarget();
      consumed = true;
      break;
    }
    return consumed || super.eventWidgetDown(i, event);
  }

  MyInputLine mInputLine;
  ActiveList mExampleList;
  ActiveListView mExampleListView;
}

class MyActiveListView extends ActiveListView {

  MyActiveListView() {
    super();
  }

  protected void itemActivated(Object item) {
    Morse.playString((String)item);
  }
  
}

class MyInputLine extends TextField {
  MyInputLine() {
    super();
  }

  public boolean eventKeyUp(char inChar, Event event) {
    if (inChar == 13) {
      String text = toString();
      if (text != null && text.length() > 0) {
	DEBUG.p("Adding text " + text);
	CW app = ((CW)Application.getCurrentApp());
	app.addItem(text);
	setText("");
      }
      return true;
    }
    else {
      return super.eventKeyUp(inChar, event);
    }
  }

}

abstract class Morse {
  static Sequence seq = null;

  static void playString(String line) {
    int len = line.length();
    try {
      // AudioManager.setDebugLevel(3);
      if (seq == null)
	seq = AudioManager.createSequence();
      seq.programBankChange(channel, ((byte) 0), ((byte)0));
      seq.setVolume(100);
    } catch (AudioException e) {
      e.printStackTrace();
    }

    try {
      int t = AudioManager.getTime();
    
      for (int i = 0; i < len; i++) {
	char c = line.charAt(i);
	if (c == ' ') {
	  t = wordSpace(t);
	} else if (Character.isDigit(c)) {
	  String code = DIGITS[c-'0'];
	  t = playCode(t, code);
	} else {
	  String code = LETTERS[c-'A'];
	  t = playCode(t, code);
	}
      }
    } catch (Exception e) {
      if (seq != null) {
	seq.destroy();
	seq = null;
	e.printStackTrace();
      }
    } 
    if (seq != null) {
      seq.unmuteChannel(channel);
      DEBUG.p("doing " + seq + ".start()");
      seq.start();
      seq = null;
    }
  }

  static int playCode(int t, String code) {
    int len = code.length();
    for (int i = 0; i < len; i++) {
      char c = code.charAt(i);
      switch(c) {
      case '*':
	t = dit(t);
	break;
      case '-':
	t = dah(t);
	break;
      }
    }
    t = charSpace(t);
    return t;
  }

  private static final int ditDuration = 50*1000;
  private static final int dahDuration = ditDuration*3;
  private static final byte note = (byte)79; // G, about 783Hz (want about 800Hz).
  private static final byte channel = 1;
  private static final byte velocity = (byte)100;
  static int dit(int t) {
    seq.noteOnWithLoad(channel, note, velocity, t);
    t += ditDuration;
    seq.noteOff(channel, note, velocity, t);
    t += ditDuration;
    return t;
  }

  static int dah(int t) {
    seq.noteOnWithLoad(channel, note, velocity, t);
    t += dahDuration;
    seq.noteOff(channel, note, velocity, t);
    t += ditDuration;
    return t;
  }

  static int charSpace(int t) {
    t += dahDuration;
    return t;
  }

  static int wordSpace(int t) {
    t += dahDuration*3;
    return t;
  }


  final static String[] LETTERS = {
    "*-",
    "-***",
    "-*-*",
    "-**",
    "*",
    "**-*",
    "--*",
    "****",
    "**",
    "*---",
    "-*-",
    "*-**",
    "--",
    "-*",
    "---",
    "*--*",
    "--*-",
    "*-*",
    "***",
    "-",
    "**-",
    "***-",
    "*--",
    "-**-",
    "-*--",
    "--**"
  };

  final static String[] DIGITS = {
    "-----",
    "*----",
    "**---",
    "***--",
    "****-",
    "*****",
    "-****",
    "--***",
    "---**",
    "----*"
  };
}
