package com.hipme.cw;

// Example use of ActiveListView derived from org.cymry.active.AListEx
// (c) 2003, 2004, 2005, Leigh L. Klotz, Jr. <klotz@graflex.org>

import danger.app.*;
import danger.util.*;
import danger.ui.ScreenWindow;
import danger.ui.TextField;
import danger.audio.Sound;
import danger.audio.Meta;
import danger.audio.Sequence;
import danger.audio.AudioManager;
import danger.audio.AudioException;

public class CW extends Application implements Commands {
  private CWWindow mWindow;

  public CW() {
    mWindow = new CWWindow();
  }

  public void resume() {
    mWindow.show();
  }

  void addItem(Object item) {
    mWindow.mExampleList.addItem(item);
  }

  public final boolean receiveEvent(Event event) {
    boolean consumed = false;
    switch (event.type) {
    case kReturnToLauncher:
      returnToLauncher();
      consumed = true;
      break;
    }
    return consumed || super.receiveEvent(event);
  }
}

class CWWindow extends ScreenWindow implements Commands {

  public CWWindow() {
    setTitle("CW Example");

    {
      mExampleList = new StdActiveList();
      mExampleList.addItem("CQ CQ");
      mExampleList.addItem("DE");
      mExampleList.addItem("WA5ZNU");
      mExampleList.addItem("R");
      mExampleList.addItem("K");
    }

    {
      mExampleListView = new MyActiveListView();
      mExampleListView.setList(mExampleList);
      mExampleListView.setPosition(10,10);
      mExampleListView.setSize(220,80);
      mExampleListView.setItemHeight(10);
      mExampleListView.setHasScrollbar(true);
      addChild(mExampleListView);
    }

    {
      mInputLine = new MyInputLine();
      mInputLine.setPosition(10, 100);
      mInputLine.setSize(220, 15);
    }

    {
      addChild(mInputLine);
      mInputLine.show();
      mExampleListView.show();
      setFocusedChild(mExampleListView);
    }
  }
  
  public boolean eventWidgetDown(int i, Event event) {
    boolean consumed = false;
    switch(i) {
    case Event.DEVICE_BUTTON_BACK:
      new Event(Application.getCurrentApp(), kReturnToLauncher).sendToTarget();
      consumed = true;
      break;
    }
    return consumed || super.eventWidgetDown(i, event);
  }

  MyInputLine mInputLine;
  StdActiveList mExampleList;
  ActiveListView mExampleListView;
}

class MyActiveListView extends ActiveListView {

  MyActiveListView() {
    super();
  }

  protected void itemActivated(Object item) {
    Morse.playString((String)item);
  }
  
}

class MyInputLine extends TextField {
  MyInputLine() {
    super();
  }

  public boolean eventKeyUp(char inChar, Event event) {
    if (inChar == 13) {
      String text = toString();
      if (text != null && text.length() > 0) {
	DEBUG.p("Adding text " + text);
	CW app = ((CW)Application.getCurrentApp());
	app.addItem(text);
	setText("");
      }
      return true;
    }
    else {
      return super.eventKeyUp(inChar, event);
    }
  }

}

abstract class Morse {
  static Sequence seq = null;

  static void playString(String line) {
    int len = line.length();
    try {
      seq = AudioManager.createSequence(true, danger.audio.Format.Type.MIDI);
      seq.loadFromRtx("four33:b=8,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p,32p");
      {
	try {
	  int t = AudioManager.getTimePlusQuarterSecond();
    
	  for (int i = 0; i < len; i++) {
	    char c = line.charAt(i);
	    if (c == ' ') {
	      t = wordSpace(t);
	    } else if (Character.isDigit(c)) {
	      t = playCode(t, DIGITS[c-'0']);
	    } else {
	      t = playCode(t, LETTERS[c-'A']);
	    }
	  }
	  seq.allNotesOff();
	} catch (Exception e) {
	  seq.allNotesOff();
	  if (seq != null) {
	    seq.destroy();
	    seq = null;
	    e.printStackTrace();
	  }
	} 
      }
      if (seq != null) {
	seq.unmuteChannel(channel);
	DEBUG.p("doing " + seq + ".start() seq.getMicrosecondLength()="+seq.getMicrosecondLength());
	seq.start();
	seq = null;
      }
    }
    catch (AudioException e) {
      e.printStackTrace();
    }
  }


  static int playCode(int t, String code) {
    int len = code.length();
    for (int i = 0; i < len; i++) {
      char c = code.charAt(i);
      switch(c) {
      case '*':
	t = dit(t);
	break;
      case '-':
	t = dah(t);
	break;
      }
    }
    t = charSpace(t);
    return t;
  }

  private static final int ditDuration = 1000*40;
  private static final int dahDuration = ditDuration*3;
  private static final byte note = (byte)74;
  private static final byte channel = 0;
  private static final byte onVelocity = (byte)200;
  private static final byte offVelocity = (byte)255;
  static int dit(int t) {
    seq.noteOnWithLoad(channel, note, onVelocity, t);
    t += ditDuration;
    seq.noteOff(channel, note, offVelocity, t);
    t += ditDuration;
    return t;
  }

  static int dah(int t) {
    seq.noteOnWithLoad(channel, note, onVelocity, t);
    t += dahDuration;
    seq.noteOff(channel, note, offVelocity, t);
    t += ditDuration;
    return t;
  }

  static int charSpace(int t) {
    return t + dahDuration;
  }

  static int wordSpace(int t) {
    return t + dahDuration*3;
  }


  final static String[] LETTERS = {
    "*-",
    "-***",
    "-*-*",
    "-**",
    "*",
    "**-*",
    "--*",
    "****",
    "**",
    "*---",
    "-*-",
    "*-**",
    "--",
    "-*",
    "---",
    "*--*",
    "--*-",
    "*-*",
    "***",
    "-",
    "**-",
    "***-",
    "*--",
    "-**-",
    "-*--",
    "--**"
  };

  final static String[] DIGITS = {
    "-----",
    "*----",
    "**---",
    "***--",
    "****-",
    "*****",
    "-****",
    "--***",
    "---**",
    "----*"
  };
}
